"""
A python utility for creating better modules/packages.
This module give the ability to quickly create and add files to different directories.
From creating directory & sub directory to managing the files that are put into those folders.
This is more of an automation tool for writing modules but can be used for other package related matters.

Example:
    >>> from cpackage import CPack, CAccess
    >>> CPack("mymodule", True, "code")
    >>> CAccess("mymodule/code", True, "example.py")
"""

__all__     = ["CPack", "Cdir", "CAccess", "CRemove"]
__author__  = "Cru1seControl <Cru1seControl.loot@gmail.com>"
__version__ = "v1.0.0"


try: #Ignore errors, just dont run this file!
    from cpackage.packaging import CPack
    from cpackage.packaging import Cdir
    from cpackage.packaging import CAccess
    from cpackage.packaging import CRemove

except ImportError as importerror:
    print(importerror)



from shutil import rmtree
import sys
import os

class _construct(object):
    # Create main module package directory
    def _pkgMain(pkg, cur=True):
        if cur:
            cpkg = os.getcwd() + "/" + pkg
        else:
            cpkg = sys.path[4] + "/" + pkg

        return cpkg

    # Create sub packages
    def _pkgSub(pkg, sub, cur=True):
        if cur:
            for sub_pkg in sub:
                spkg = os.getcwd() + "/" + pkg + "/" + sub_pkg
                yield spkg
        else:
            for sub_pkg in sub:
                spkg = sys.path[4] + "/" + pkg + "/" + sub_pkg
                yield spkg

    # Check if package name exists & raise FileExistsError
    def _exists(pkg, main=True):
        if main:
            if os.path.exists(pkg):
                raise FileExistsError("Main pkg {} already exists".format(pkg))
        else:
            if os.path.exists(pkg):
                raise FileExistsError("Sub package {} already exists".format(pkg))

# Create package name and subpackages
def CPack(pkg, cur_dir=True, *sub_packages):
    if cur_dir:
        main_current = _construct._pkgMain(pkg, True)

        _construct._exists(main_current, True)
        os.mkdir(main_current)

        for sub_current in _construct._pkgSub(pkg, sub_packages, True):
            _construct._exists(sub_current, False)
            os.mkdir(sub_current)

    else:
        main_site = _construct._pkgMain(pkg, False)

        _construct._exists(main_site, True)
        os.mkdir(main_site)

        for sub_site in _construct._pkgSub(pkg, sub_packages, False):
            _construct._exists(sub_site, False)
            os.mkdir(sub_site)

# Show all aviable packages in dictionary form e.g {"mymodule": ["sub1", "sub2"]}
def Cdir(pkg, cur_dir=True):
    items = []
    current = os.getcwd() + "/" + pkg
    site = sys.path[4] + "/" + pkg
    if cur_dir:
        for pkg_dir in os.listdir(current):
            items.append(pkg_dir)
            items.sort(reverse=True)
            items.reverse()
    else:
        for pkg_dir in os.listdir(site):
            items.append(pkg_dir)
            items.sort(reverse=True)
            items.reverse()

    return {pkg: items}

# Access sub directorys in the specfied package
def CAccess(pkg, cur_dir=True, *fd):
    if cur_dir:
        for sub_pkg in _construct._pkgSub(pkg, fd, True):
            _construct._exists(sub_pkg, False)
            os.mknod(sub_pkg)

    else:
        for sub_pkg in _construct._pkgSub(pkg, fd, False):
            _construct._exists(sub_pkg, False)
            os.mknod(sub_pkg)

# Remove the package tree 'good if you want everything gone'
def CRemove(pkg, cur_dir=True):
    if cur_dir:
        pl = os.getcwd() + "/" + pkg
    else:
        pl = sys.path[4] + "/" + pkg

    rmtree(pl)



try:
    import argparse
    import os
    import sys
    from cpackage.packaging import Cdir, CRemove

except ImportError as importerror:
    print(importerror)

parser = argparse.ArgumentParser()
parser.add_argument("-pkg", help="Package name to create")
parser.add_argument("-sub", help="Sub package names to create", nargs="+", type=str)
parser.add_argument("-listdir", help="List directory and sub packages")
parser.add_argument("-access", help="Create files within a package")
parser.add_argument("-remove", help="Remove entire package tree")

parser.add_argument("--site", help="Use python site packages directory", action="store_true")
parser.add_argument("--current", help="Use current working directory", action="store_true")

args = parser.parse_args()

class _toolConstruct(object):
    def isCurrent():
        if args.current:
            return True
        elif args.site:
            return False

    # has potential future use 
    def subTup():
        argsubLst = []
        if args.sub:
            for arg in args.sub:
                argsubLst.append(arg)
        return argsubLst

def tool():
    # get package argument and create directory
    if args.pkg:
        if _toolConstruct.isCurrent():
            directory = os.getcwd() + "/" + args.pkg
            os.mkdir(directory)
        else:
            directory = sys.path[4] + "/" + args.pkg
            os.mkdir(directory)

    # create package and sub packages
    if args.pkg and args.sub:
        if _toolConstruct.isCurrent():
            for sub_dir in args.sub:
                sub_directory = os.getcwd() + "/" + args.pkg + "/" + sub_dir
                os.mkdir(sub_directory)
        else:
            for sub_dir in args.sub:
                sub_directory = sys.path[4] + "/" + args.pkg + "/" + sub_dir
                os.mkdir(sub_directory)

    # list the packages in a dictionary
    if args.listdir:
        if _toolConstruct.isCurrent():
            print(Cdir(args.listdir, True))
        else:
            print(Cdir(args.listdir, False))

    # access a package and write a file
    if args.access:
        if _toolConstruct.isCurrent():
            #for main_dir in args.access:
            main_directory = os.getcwd() + "/" + args.access
            os.mknod(main_directory)

        else:
            sub_directory = sys.path[4] + "/" + args.access
            os.mknod(sub_directory)

    # remove an entire package tree
    if args.remove:
        if _toolConstruct.isCurrent():
            CRemove(args.remove, True)
        else:
            CRemove(args.remove, False)

tool()




from distutils.core import setup

setup(
    name="cpackage",
    packages=["cpackage"],
    version="v1.0.2",
    license="MIT",
    description="""A python utility for creating better modules/packages.
    This module give the ability to quickly create and add files to different directories.
    From creating directory & sub directory to managing the files that are put into those folders.
    This is more of an automation tool for writing modules but can be used for other package related matters.""",
    author="Cru1seControl",
    author_email="Cru1seControl.loot@gmail.com",
    url="https://github.com/Cru1seControl",
    download_url="https://github.com/Cru1seControl/python-cpackage",
    keywords=["packaging", "filesystem", "tool"],
    classifiers=[
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "Topic :: Software Development :: Build Tools",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 3.6"
    ]

)
